/*****************************************************************
******************************************************************
***                                                            ***
***        (C)Copyright 2008, American Megatrends Inc.         ***
***                                                            ***
***                    All Rights Reserved                     ***
***                                                            ***
***       5555 Oakbrook Parkway, Norcross, GA 30093, USA       ***
***                                                            ***
***                     Phone 770.246.8600                     ***
***                                                            ***
******************************************************************
******************************************************************
******************************************************************
*
* Filename: libipmi_StorDevice.c
*
* Description: Contains implementation of NetFn Storage
*   specific IPMI command functions
*
* Author: Anurag Bhatia
*
******************************************************************/
#include "libipmi_session.h"
#include "libipmi_errorcodes.h"
#include "libipmi_StorDevice.h"
//#include "libipmi_AMIOEM.h"

#include <stdlib.h>
#include <string.h>

#include "goahead.h"
#include "com_IPMI_App.h"

/* Helper function */
uint8_t libipmi_GetBits(uint8_t orig, uint8_t startbit, uint8_t endbit)
{
	uint8_t  temp = orig;
	uint8_t  mask = 0x00;
	int i;
	for(i=startbit;i>=endbit;i--)
	{
		mask = mask | (1 << i);
	}

	return (temp & mask);
}

/*****************************************************************************
	System Event Log
******************************************************************************/

uint16_t IPMICMD_GetSELAllocationInfo(IPMI20_UDS_SESSION_T *pUDSSession,
					SELAllocInfo_T *pResGetSELAllocInfo,
					int timeout)
{
	uint16_t	wRet;
	uint32_t	dwResLen;
	uint8_t	DummyReqData[8];

	dwResLen = sizeof(SELAllocInfo_T);
	wRet = LIBIPMI_Send_RAW_IPMI2_0_Command(pUDSSession,  
						NETFNLUN_IPMI_STORAGE, CMD_GET_SEL_ALLOCATION_INFO,
						(uint8_t*)DummyReqData, 0,
						(uint8_t *)pResGetSELAllocInfo, &dwResLen,
						timeout);
	return wRet;
}

/* This function reserves SEL and returns SEL reservation ID */
uint16_t IPMICMD_ReserveSEL(IPMI20_UDS_SESSION_T *pUDSSession,
					ReserveSELRes_T* pReserveSelRes,
					int timeout)
{
	uint16_t	wRet = 0;
	uint32_t	dwResLen = 0;

	memset(pReserveSelRes, 0, sizeof(ReserveSELRes_T));
	wRet = LIBIPMI_Send_RAW_IPMI2_0_Command(pUDSSession,  
						NETFNLUN_IPMI_STORAGE, CMD_RESERVE_SEL,
						NULL, 0,
						(uint8_t *)pReserveSelRes, &dwResLen,
						timeout);

	/* If ReserveSEL command is not implemented, return 0x0000 as reservation ID */
	if ( pReserveSelRes->CompletionCode == CC_INV_CMD )
	{
		pReserveSelRes->ReservationID = 0x0000;
		return 0;
	}

	if(wRet != 0) 
	{
		printf("Could not get SEL reservation!!\n");
		return wRet;
	}
	return 0;
}

uint16_t IPMC_ReserveSEL(IPMI20_UDS_SESSION_T *pUDSSession, uint8_t IpmbAddr,
					ReserveSELRes_T* pReserveSelRes,
					int timeout)
{
	int wRet = 0;
	uint32_t	dwResLen;
	uint8_t 	Req[50] = {0};
	uint8_t		Res[50] = {0};

	Req[0] = 0x40;	//Track | channel 0
	Req[1] = IpmbAddr;
	Req[2] = NETFNLUN_IPMI_STORAGE;
	Req[3] = 0x100 - (Req[1] + Req[2])&0xff;	//checksum1
	Req[4] = 0x20;
	Req[5] = 0x10;
	Req[6] = CMD_RESERVE_SEL;	
	Req[7] = 0x100 - (Req[4] + Req[5] + Req[6])&0xff;

	wRet = LIBIPMI_Send_RAW_IPMI2_0_Command(pUDSSession,  
							NETFNLUN_IPMI_APP, CMD_SEND_MSG,
							Req, 8,
							Res,  &dwResLen,
							timeout);

	memcpy(pReserveSelRes, &Res[6], sizeof(ReserveSELRes_T));
	/* If ReserveSEL command is not implemented, return 0x0000 as reservation ID */
	if ( pReserveSelRes->CompletionCode == CC_INV_CMD )
	{
		pReserveSelRes->ReservationID = 0x0000;
		return 0;
	}

	if(wRet != 0) 
	{
		printf("Could not get SEL reservation!!\n");
		return wRet;
	}
	return 0;
}

/* This function gets an entry from SEL. */
uint16_t IPMICMD_GetSELEntry(IPMI20_UDS_SESSION_T *pUDSSession,
				GetSELReq_T* pGetSELReqData,
				uint8_t* pGetSELResData,
				uint32_t ResDataLen,
				int timeout)
{
	uint16_t wRet = 0;

	wRet = LIBIPMI_Send_RAW_IPMI2_0_Command(pUDSSession,  
						NETFNLUN_IPMI_STORAGE, CMD_GET_SEL_ENTRY,
						(uint8_t*)pGetSELReqData, sizeof(GetSELReq_T),
						(uint8_t *)pGetSELResData, &ResDataLen,
						timeout);
	return wRet;
}

uint16_t IPMI_GetSELEntry(IPMI20_UDS_SESSION_T *pUDSSession,
				GetSELReq_T* pGetSELReqData,
				SELEventRecord_T* pSELEventData,
				uint16_t *nextRecID,
				int timeout)
{
	uint16_t 	wRet = 0;
	uint8_t 	Res[50];
	uint32_t	dwResLen;

	wRet = LIBIPMI_Send_RAW_IPMI2_0_Command(pUDSSession,  
						NETFNLUN_IPMI_STORAGE, CMD_GET_SEL_ENTRY,
						(uint8_t*)pGetSELReqData, sizeof(GetSELReq_T),
						Res, &dwResLen,
						timeout);
	// printf("---> Res ");
	// int i;
	// for(i=0;i<17;i++)
	// 	printf("%02x ", Res[i]);
	// printf("\n");
	if((wRet != 0) || (Res[0] != 0))
	{
		printf("IPMI_GetSELEntry failed\n");
		*nextRecID = 0xffff;
		memset(pSELEventData, 0, sizeof(SELEventRecord_T));
		return wRet;
	}

	*nextRecID = (Res[2]<<8) | Res[1];
	memcpy(pSELEventData, &Res[3], sizeof(SELEventRecord_T));

	return 0;
}

uint16_t IPMC_GetSELEntry(IPMI20_UDS_SESSION_T *pUDSSession, uint8_t IpmbAddr,
				GetSELReq_T* pGetSELReqData,
				SELEventRecord_T* pSELEventData,
				uint16_t *nextRecID,
				int timeout)
{
	int wRet = 0;
	uint32_t	dwResLen;
	uint8_t 	Req[50] = {0};
	uint8_t		Res[50] = {0};

	Req[0] = 0x40;	//Track | channel 0
	Req[1] = IpmbAddr;
	Req[2] = NETFNLUN_IPMI_STORAGE;
	Req[3] = 0x100 - (Req[1] + Req[2])&0xff;	//checksum1
	Req[4] = 0x20;
	Req[5] = 0x10;
	Req[6] = CMD_GET_SEL_ENTRY;	
	Req[7] = pGetSELReqData->ReservationID&0xff;
	Req[8] = (pGetSELReqData->ReservationID>>8)&0xff;
	Req[9] = pGetSELReqData->RecID&0xff;
	Req[10] = (pGetSELReqData->RecID>>8)&0xff;
	Req[11] = pGetSELReqData->Offset;
	Req[12] = pGetSELReqData->Size;
	Req[13] = 0x100 - (Req[4] + Req[5] + Req[6] + Req[7] + Req[8] + Req[9] + Req[10] + Req[11] + Req[12])&0xff;

	wRet = LIBIPMI_Send_RAW_IPMI2_0_Command(pUDSSession,  
							NETFNLUN_IPMI_APP, CMD_SEND_MSG,
							Req, 14,
							Res,  &dwResLen,
							timeout);
	if((wRet != 0) || (Res[6] != 0))
	{
		printf("IPMI_GetSELEntry failed\n");
		*nextRecID = 0xffff;
		memset(pSELEventData, 0, sizeof(SELEventRecord_T));
		return wRet;
	}

	*nextRecID = (Res[8]<<8) | Res[7];
	memcpy(pSELEventData, &Res[9], sizeof(SELEventRecord_T));

	return 0;
}

/* This function adds an entry into SEL. */
uint16_t IPMICMD_AddSELEntry(IPMI20_UDS_SESSION_T *pUDSSession,
				SELEventRecord_T* pSELReqData,
				AddSELRes_T* pSELResData,
				int timeout)
{
	uint16_t wRet = 0;
	uint32_t	dwResLen;

	dwResLen = sizeof(AddSELRes_T);
	wRet = LIBIPMI_Send_RAW_IPMI2_0_Command(pUDSSession,  
						NETFNLUN_IPMI_STORAGE, CMD_ADD_SEL_ENTRY,
						(uint8_t*)pSELReqData, sizeof(SELEventRecord_T),
						(uint8_t *)pSELResData, &dwResLen,
						timeout);
	return wRet;
}

/* This function deletes an entry from SEL. */
uint16_t IPMICMD_DeleteSELEntry(IPMI20_UDS_SESSION_T *pUDSSession,
				uint16_t RecordID,
				int timeout)
{
	uint16_t			wRet = 0;
	uint32_t			dwResLen = 0;
	ReserveSELRes_T	ReserveSelRes;
	DeleteSELReq_T	DeleteSELReqData;
	DeleteSELRes_T	DeleteSELResData;

	memset(&ReserveSelRes, 0, sizeof(ReserveSelRes));
	memset(&DeleteSELReqData, 0, sizeof(DeleteSELReqData));
	memset(&DeleteSELResData, 0, sizeof(DeleteSELResData));

	//first do a SEL reservation
	dwResLen = sizeof(ReserveSELRes_T);
	wRet = LIBIPMI_Send_RAW_IPMI2_0_Command(pUDSSession,  
						NETFNLUN_IPMI_STORAGE, CMD_RESERVE_SEL,
						NULL, 0,
						(uint8_t *)&ReserveSelRes, &dwResLen,
						timeout);
	if ( (wRet != 0) && (ReserveSelRes.CompletionCode != CC_INV_CMD) )
	{
		printf("Could not get SEL reservation while trying to delete event log!!\n");
		return wRet;
	}

	if (ReserveSelRes.CompletionCode == CC_INV_CMD)
	{
		// Reservation ID should be set as 0x0000 for implementations that don't
		// implement ReserveSEL command
		DeleteSELReqData.ReservationID = 0x0000;
	}
	else
	{
		DeleteSELReqData.ReservationID = ReserveSelRes.ReservationID;
	}

	DeleteSELReqData.RecID = RecordID;

	wRet = LIBIPMI_Send_RAW_IPMI2_0_Command(pUDSSession,  
						NETFNLUN_IPMI_STORAGE, CMD_DELETE_SEL_ENTRY,
						(uint8_t*)&DeleteSELReqData, sizeof(DeleteSELReqData),
						(uint8_t *)&DeleteSELResData, &dwResLen,
						timeout);
	return wRet;
}

uint16_t IPMICMD_ClearSEL(IPMI20_UDS_SESSION_T *pUDSSession,
				int timeout)
{
	uint16_t	wRet;
	uint32_t	dwResLen;
	ClearSELReq_T ClearSelReq;
	ClearSELRes_T ClearSelRes;
	ReserveSELRes_T ReserveSelRes;

	uint8_t DummyReqData[8];

	//first do a SEL reservation
 	dwResLen = sizeof(ReserveSELRes_T);
	wRet = LIBIPMI_Send_RAW_IPMI2_0_Command(pUDSSession,  
						NETFNLUN_IPMI_STORAGE, CMD_RESERVE_SEL,
						(uint8_t*)DummyReqData, 0,
						(uint8_t *)&ReserveSelRes, &dwResLen,
						timeout);

	if ( (wRet != 0) && (ReserveSelRes.CompletionCode != CC_INV_CMD) )
	{
		printf("Could not get SEL reservation while trying to clear event log!!\n");
		return wRet;
	}

	if (ReserveSelRes.CompletionCode == CC_INV_CMD)
	{
		// Reservation ID should be set as 0x0000 for implementations that don't
		// implement ReserveSEL command
		ClearSelReq.ReservationID = 0x0000;
	}
	else
	{
		ClearSelReq.ReservationID = ReserveSelRes.ReservationID;
	}

	ClearSelReq.CLR[0] = 'C';
	ClearSelReq.CLR[1] = 'L';
	ClearSelReq.CLR[2] = 'R';
	ClearSelReq.InitOrStatus = 0xAA;

	dwResLen = sizeof(ClearSELRes_T);
	wRet = LIBIPMI_Send_RAW_IPMI2_0_Command(pUDSSession,  
						NETFNLUN_IPMI_STORAGE, CMD_CLEAR_SEL,
						(uint8_t*)&ClearSelReq, sizeof(ClearSELReq_T),
						(uint8_t *)&ClearSelRes, &dwResLen,
						timeout);
	return wRet;
}

uint16_t IPMC_ClearSEL(IPMI20_UDS_SESSION_T *pUDSSession, uint8_t IpmbAddr, 
				int timeout)
{
	uint16_t 	ReservationID = 0;

	int wRet = 0;
	uint32_t	dwResLen;
	uint8_t 	Req[50] = {0};
	uint8_t		Res[50] = {0};

	//first do a SEL reservation
	Req[0] = 0x40;	//Track | channel 0
	Req[1] = IpmbAddr;
	Req[2] = NETFNLUN_IPMI_STORAGE;
	Req[3] = 0x100 - (Req[1] + Req[2])&0xff;	//checksum1
	Req[4] = 0x20;
	Req[5] = 0x10;
	Req[6] = CMD_RESERVE_SEL;	
	Req[7] = 0x100 - (Req[4] + Req[5] + Req[6])&0xff;

	wRet = LIBIPMI_Send_RAW_IPMI2_0_Command(pUDSSession,  
							NETFNLUN_IPMI_APP, CMD_SEND_MSG,
							Req, 8,
							Res,  &dwResLen,
							timeout);

	if ( (wRet != 0)  )
	{
		printf("Could not get SEL reservation while trying to clear event log!!\n");
		return wRet;
	}

	if (Res[6] == CC_INV_CMD)
	{
		// Reservation ID should be set as 0x0000 for implementations that don't
		// implement ReserveSEL command
		ReservationID = 0x0000;

	}
	else
	{
		ReservationID = (Res[8]<<8) | Res[7];
	}

	Req[0] = 0x40;	//Track | channel 0
	Req[1] = IpmbAddr;
	Req[2] = NETFNLUN_IPMI_STORAGE;
	Req[3] = 0x100 - (Req[1] + Req[2])&0xff;	//checksum1
	Req[4] = 0x20;
	Req[5] = 0x10;
	Req[6] = CMD_CLEAR_SEL;	
	Req[7] = ReservationID&0xff;
	Req[8] = (ReservationID>>8)&0xff;
	Req[9] = 'C';
	Req[10] = 'L';
	Req[11] = 'R';
	Req[12] = 0xAA;	//InitOrStatus
	Req[13] = 0x100 - (Req[4] + Req[5] + Req[6] + Req[7] + Req[8] + Req[9] + Req[10] + Req[11] + Req[12])&0xff;

	wRet = LIBIPMI_Send_RAW_IPMI2_0_Command(pUDSSession,  
							NETFNLUN_IPMI_APP, CMD_SEND_MSG,
							Req, 14,
							Res,  &dwResLen,
							timeout);

	if ( (wRet != 0) || (Res[6] != 0) )
	{
		printf("Clear event log failed!\n");
		return wRet;
	}

	return wRet;
}

/**************************************************************************************
*****************************Higher level commands*****************************
**************************************************************************************/
/* This function returns total number of SEL entries present in the system at that time */
uint16_t LIBIPMI_HL_GetNumOfSELEntries(IPMI20_UDS_SESSION_T *pUDSSession,
					uint16_t* nNumOfSELEntries,
					int timeout)
{
	SELInfo_T SELInfo;
	uint16_t	wRet;
	uint32_t	dwResLen;
	uint8_t	DummyReqData[8];

	dwResLen = sizeof(SELInfo_T);
	wRet = LIBIPMI_Send_RAW_IPMI2_0_Command(pUDSSession,  
						NETFNLUN_IPMI_STORAGE, CMD_GET_SEL_INFO,
						(uint8_t*)DummyReqData, 0,
						(uint8_t *)&SELInfo, &dwResLen,
						timeout);
	if((wRet != 0) || (SELInfo.CompletionCode != 0))
	{
		printf("LIBIPMI_HL_GetNumOfSELEntries: Error getting SEL info\n");
		*nNumOfSELEntries = 0;
		return wRet;
	}
	*nNumOfSELEntries =  (SELInfo.RecCt);
	return 0;
}

uint16_t IPMC_GetNumOfSELEntries(IPMI20_UDS_SESSION_T *pUDSSession, uint8_t IpmbAddr, 
					uint16_t* nNumOfSELEntries,
					int timeout)
{
	SELInfo_T *pSELInfo;
	int wRet = 0;
	uint32_t	dwResLen;
	uint8_t 	Req[50] = {0};
	uint8_t		Res[50] = {0};

	Req[0] = 0x40;	//Track | channel 0
	Req[1] = IpmbAddr;
	Req[2] = NETFNLUN_IPMI_STORAGE;
	Req[3] = 0x100 - (Req[1] + Req[2])&0xff;	//checksum1
	Req[4] = 0x20;
	Req[5] = 0x10;
	Req[6] = CMD_GET_SEL_INFO;	
	Req[7] = 0x100 - (Req[4] + Req[5] + Req[6])&0xff;

	wRet = LIBIPMI_Send_RAW_IPMI2_0_Command(pUDSSession,  
							NETFNLUN_IPMI_APP, CMD_SEND_MSG,
							Req, 8,
							Res,  &dwResLen,
							timeout);

	if ( (wRet != 0) || (Res[6] != 0) )
	{
		printf("Get SEL Number failed!\n");
		*nNumOfSELEntries = 0;
		return wRet;
	}

	pSELInfo = (SELInfo_T *)&Res[6];
	*nNumOfSELEntries =  (pSELInfo->RecCt);

	return 0;
}


/* This function returns total number of SEL entries present in the system at that time.
   IMPORTANT:
   Before you call this function, call LIBIPMI_HL_GetNumOfSELEntries() to get the number of
   SEL entries present in the system, then allocate a buffer size of ((Num of Entries) * SELEventRecord_T)
   to hold the total number of SEL entries. Pass the pointer to this buffer as pSELEntriesBuffer when
   calling this function.
   Alternatively, you can call LIBIPMI_HL_GetMaxPossibleSELEntries() to get the maximum possible
   number of SEL entries present in the system at any given time, and allocate the buffer to hold those
   many entries. It's just that this approach would allocate a bigger buffer size. */
uint16_t LIBIPMI_HL_GetAllSELEntries(IPMI20_UDS_SESSION_T *pUDSSession,
					SELEventRecord_T *pSELEntriesBuffer,
					int timeout)
{
	GetSELReq_T GetSELReqData = {0x00,0x00,0x00,0xFF}; // Request packet to get the very
							// first SEL record and complete record,
													// starting at offset 0 of each record.
	uint8_t *GetSELResData = NULL;
	SELEventRecord_T *ptrToSELRecord  = NULL;
	uint32_t DataLen = 0;
	uint16_t wRet = 0;

	/* We want to read full records */
	DataLen = sizeof(GetSELRes_T) + sizeof(SELEventRecord_T);
	GetSELResData = (uint8_t *) malloc(DataLen);
	if (NULL == GetSELResData)
	{
		return LIBIPMI_E_NO_ENOUGH_MEMORY;
	}

	/* Fill in the Record data first */
	while(1)
	{
		wRet = IPMICMD_GetSELEntry(pUDSSession, &GetSELReqData, GetSELResData, DataLen,timeout);
		if(wRet == LIBIPMI_E_SUCCESS)
		{
			ptrToSELRecord = (SELEventRecord_T *) (GetSELResData + sizeof(GetSELRes_T));
			memcpy(pSELEntriesBuffer,ptrToSELRecord,sizeof(SELEventRecord_T));
			pSELEntriesBuffer++;
			GetSELReqData.RecID = ((GetSELRes_T *) GetSELResData)->NextRecID;
			if(  ((GetSELRes_T *) GetSELResData)->NextRecID == 0xFFFF)
			{
				/* End of SEL reached. */
				break;
			}
		}
		else
		{
			printf("LIBIPMI_HL_GetAllSELEntries: Error getting SEL entry\n");
			break;
		}
	}

	free(GetSELResData);
	return wRet;
}



					
uint16_t LIBIPMI_HL_GetAllSensorInfo(IPMI20_UDS_SESSION_T *pUDSSession,
					uint8_t **SDRBuff,
					int *SDRCount,
					int *MaxSDRLen,
					int timeout)
{
	int wRet = 0;
	SDRRepositoryAllocInfo_T SDRAllocationInfo;
	SDRRepositoryInfo_T SDRReposInfo;
    uint8_t *pSDRBuff=NULL;
    	int MaxLen = 0;
    	int SDRCnt = 0;

	/* Get max buffer size to be allocated to read all SDRs in the system */
	wRet = IPMICMD_GetSDRRepositoryAllocInfo(pUDSSession, &SDRAllocationInfo, timeout);
	if( wRet != 0 )
	{
		printf("Error getting SDR Repository Allocation Info\n");
		return wRet;
	}
	
	/* Get repository information */
	wRet = IPMICMD_GetSDRRepositoryInfo(pUDSSession, &SDRReposInfo,timeout);
	if( wRet != 0 )
	{
		printf("Error getting SDR Repository Info\n");
		return wRet;
	}

	/* Determine max SDR entry length */
	MaxLen = SDRAllocationInfo.MaxRecSize *  ( SDRAllocationInfo.AllocUnitSize );

	/* Determine total number of SDRs stored */
	SDRCnt =  ( SDRReposInfo.RecCt );

	/* Allocate the buffer */
	pSDRBuff = (uint8_t *) malloc(MaxLen * SDRCnt);
	if (NULL == pSDRBuff)
	{
		return LIBIPMI_E_NO_ENOUGH_MEMORY;
	}


	/* Call the function to read all the SDRs into this buffer */
	wRet = Storlead_GetAllSDRs(pUDSSession, 0x20, 
                                     pSDRBuff,(MaxLen * SDRCnt),
                                     timeout*4);
	*MaxSDRLen=MaxLen;
	*SDRCount=SDRCnt;
	*SDRBuff=pSDRBuff;

	return wRet;
}

uint16_t LIBIPMI_HL_GetSensorName(IPMI20_UDS_SESSION_T *pUDSSession,
					uint8_t SlaveAdd,
					uint8_t LUNid,
					uint8_t SensorNum,
					uint8_t *SensorName,
					int   timeout)
{
	int wRet = 0;
	SDRRepositoryAllocInfo_T SDRAllocationInfo;
	SDRRepositoryInfo_T SDRReposInfo;
    	int MaxSDRLen = 0;
    	int i,SDRCount = 0;
	uint8_t *pSDRBuff = NULL;
	SDRHeader *pSDRHeader = NULL;
	uint8_t *pSDRStart = NULL;

	/* Get max buffer size to be allocated to read all SDRs in the system */
	wRet = IPMICMD_GetSDRRepositoryAllocInfo(pUDSSession, &SDRAllocationInfo, timeout);
	if( wRet != 0 )
	{
		printf("Error getting SDR Repository Allocation Info\n");
		return wRet;
	}
	/* Get repository information */
	wRet = IPMICMD_GetSDRRepositoryInfo(pUDSSession, &SDRReposInfo,timeout);
	if( wRet != 0 )
	{
		printf("Error getting SDR Repository Info\n");
		return wRet;
	}

	/* Determine max SDR entry length */
	MaxSDRLen = SDRAllocationInfo.MaxRecSize *  ( SDRAllocationInfo.AllocUnitSize );

	/* Determine total number of SDRs stored */
	SDRCount =  ( SDRReposInfo.RecCt );

	/* Allocate the buffer */
	pSDRBuff = (uint8_t *) malloc(MaxSDRLen * SDRCount);
	if (NULL == pSDRBuff)
	{
		return LIBIPMI_E_NO_ENOUGH_MEMORY;
	}
	pSDRStart = pSDRBuff;

	/* Call the function to read all the SDRs into this buffer */
	wRet = Storlead_GetAllSDRs(pUDSSession, 0x20,
                                     pSDRBuff,(MaxSDRLen * SDRCount),
                                     timeout*4);

	for( i = 0; i < SDRCount; i++)
	{
		pSDRHeader = (SDRHeader *) pSDRBuff;
		if((pSDRHeader->RecordType >= 0x01) && (pSDRHeader->RecordType <= 0x03))
		{
			/* Compare SlaveAdd and LUNid fields */
			uint8_t *pTmp = pSDRBuff + sizeof(SDRHeader);
			uint8_t len = 0;

			//if((SlaveAdd == *pTmp) && (LUNid == *(pTmp+1)))
			//{
				/* Now compare the sensor number */
				if(SensorNum == *(pTmp+2))
				{
					/* Get the Name and return */
					if(pSDRHeader->RecordType == 0x01)
						pTmp = pSDRBuff + sizeof(SDRHeader) + 42;
					else if(pSDRHeader->RecordType == 0x02)
						pTmp = pSDRBuff + sizeof(SDRHeader) + 26;
					else
						pTmp = pSDRBuff + sizeof(SDRHeader) + 11;

					len = GetBits(*pTmp,5,0);
					if(len)
					{
						pTmp++;
						memcpy(SensorName,pTmp,len);
						SensorName[len] = 0;
					}
					else
						strcpy((char*)SensorName,(char*)"\0");
					break; //Sensorname is obtained for given sensornumber. So no Need to check remaining Sensor.
				}
			//}
		}
		pSDRBuff += MaxSDRLen;
	}
	free(pSDRStart);
	return wRet;
}

/*****************************************************************************
	Field Replacement Unit (FRU)
******************************************************************************/
uint16_t IPMICMD_GetFRUInventoryAreaInfo(IPMI20_UDS_SESSION_T *pUDSSession,
				FRUInventoryAreaInfoReq_T *pReqFRUInventoryAreaInfo,
				FRUInventoryAreaInfoRes_T *pResFRUInventoryAreaInfo,
				int timeout)
{
	uint16_t	wRet,tmp;
	uint32_t	dwResLen;

	dwResLen = sizeof(FRUInventoryAreaInfoRes_T);
	wRet = LIBIPMI_Send_RAW_IPMI2_0_Command(pUDSSession,  
						NETFNLUN_IPMI_STORAGE, CMD_FRU_INVENTORY_AREA_INFO,
						(uint8_t *)pReqFRUInventoryAreaInfo, sizeof(FRUInventoryAreaInfoReq_T),
						(uint8_t *)pResFRUInventoryAreaInfo, &dwResLen,
						timeout);
	/* Do we need to do the following??? I guess so...test and make sure */
	tmp =  (pResFRUInventoryAreaInfo->Size);
	pResFRUInventoryAreaInfo->Size = tmp;
	return wRet;
}

uint16_t IPMICMD_ReadFRUData(IPMI20_UDS_SESSION_T *pUDSSession,
				FRUReadReq_T* pFRUReadReqData,
				FRUReadRes_T* pFRUReadResData,
				int timeout)
{
	uint16_t wRet = 0;
	uint32_t	dwResLen;

	dwResLen = sizeof(FRUReadRes_T) + (pFRUReadReqData->CountToRead);
	wRet = LIBIPMI_Send_RAW_IPMI2_0_Command(pUDSSession,  
						NETFNLUN_IPMI_STORAGE, CMD_READ_FRU_DATA,
						(uint8_t *)pFRUReadReqData, sizeof(FRUReadReq_T),
						(uint8_t *)pFRUReadResData, &dwResLen,
						timeout);
	return wRet;
}

uint16_t IPMICMD_WriteFRUData(IPMI20_UDS_SESSION_T *pUDSSession,
				FRUWriteReq_T* pFRUWriteReqData,
				uint32_t ReqDataLen /* Bytes to be written + sizeof(FRUWriteReq_T) */,
				FRUWriteRes_T* pFRUWriteResData,
				int timeout)
{
	uint16_t wRet = 0;
	uint32_t	dwResLen;

	dwResLen = sizeof(FRUWriteRes_T);
	wRet = LIBIPMI_Send_RAW_IPMI2_0_Command(pUDSSession,  
						NETFNLUN_IPMI_STORAGE, CMD_WRITE_FRU_DATA,
						(uint8_t *)pFRUWriteReqData, ReqDataLen,
						(uint8_t *)pFRUWriteResData, &dwResLen,
						timeout);
	return wRet;
}

uint16_t LIBIPMI_HL_GetAllFRUDevices(IPMI20_UDS_SESSION_T *pUDSSession,
					uint8_t *pFRUIDs,
					uint8_t *pFRUNames,
					uint8_t *nFRUs,
					int timeout)
{
	int wRet = 0;
	SDRRepositoryAllocInfo_T SDRAllocationInfo;
	SDRRepositoryInfo_T SDRReposInfo;
	uint8_t *pSDRBuff = NULL;
    	int MaxSDRLen = 0;
    	int i,SDRCount = 0;
	SDRHeader *pSDRHeader = NULL;
	uint8_t *pSDRStart = NULL;

	*nFRUs = 0;

	/* Get max buffer size to be allocated to read all SDRs in the system */
	wRet = IPMICMD_GetSDRRepositoryAllocInfo(pUDSSession, &SDRAllocationInfo, timeout);
	if( wRet != 0 )
	{
		printf("Error getting SDR Repository Allocation Info\n");
		return wRet;
	}
	/* Get repository information */
	wRet = IPMICMD_GetSDRRepositoryInfo(pUDSSession, &SDRReposInfo,timeout);
	if( wRet != 0 )
	{
		printf("Error getting SDR Repository Info\n");
		return wRet;
	}

	/* Determine max SDR entry length */
	MaxSDRLen = SDRAllocationInfo.MaxRecSize *  ( SDRAllocationInfo.AllocUnitSize );

//	printf("MaxSDRLen = %d\n",MaxSDRLen);
	/* Determine total number of SDRs stored */
	SDRCount =  ( SDRReposInfo.RecCt );
//	printf("SDRCount = %d\n",SDRCount);

	/* Allocate the buffer */
	pSDRBuff = (uint8_t *) malloc(MaxSDRLen * SDRCount);
	if (NULL == pSDRBuff)
	{
		return LIBIPMI_E_NO_ENOUGH_MEMORY;
	}
	memset(pSDRBuff,0,(MaxSDRLen*SDRCount));
	pSDRStart = pSDRBuff;

	/* Call the function to read all the SDRs into this buffer */
	wRet = Storlead_GetAllSDRs(pUDSSession, 0x20,
                                     pSDRBuff,(MaxSDRLen * SDRCount),
                                     timeout*4);
	if(wRet != 0)
	{
		printf("Error getting All SDRs\n");
		return wRet;
	}
	/* Search for all the SDRs of type '0x11' and '0x12' */
	/* Check SDR type '0x12' to see if FRU device is supported.
	   If yes, then FRU device id = 0.
	 	OR
	   Check SDR type '0x11' to get FRU device id.

	   Store these device ids in FRUDeviceIDs[].
	*/
	for( i = 0; i < SDRCount; i++)
	{
		uint32_t len = 0;

		pSDRHeader = (SDRHeader *) pSDRBuff;
		if(pSDRHeader->RecordType == 0x11)
		{
			uint8_t *pTmp = pSDRBuff + sizeof(SDRHeader) + 1;
			pFRUIDs[*nFRUs] = *pTmp;

//			printf("FRU Id = %d\n",pFRUIDs[*nFRUs]);
			/* Get device name now */
			pTmp = pSDRBuff + sizeof(SDRHeader) + 10;
			len = GetBits(*pTmp,5,0);
			if(len)
			{
				pTmp++;
				memcpy(pFRUNames,pTmp,len);
				pFRUNames[len] = 0;
			}
			else
				strcpy((char*)pFRUNames,(char*)"\0");

//			printf("libipmi: FRUName = %s\n",pFRUNames);
			pFRUNames += 33;
			(*nFRUs)++;
		}
		else if(pSDRHeader->RecordType == 0x12)
		{
			uint8_t *pTmp = pSDRBuff + sizeof(SDRHeader) + 3;
			if(GetBits(*pTmp,3,3))
			{
				pFRUIDs[*nFRUs] = 0;
//				printf("FRU Id = %d\n",pFRUIDs[*nFRUs]);
				/* Get device name now */
				pTmp = pSDRBuff + sizeof(SDRHeader) + 10;
				len = GetBits(*pTmp,5,0);
				if(len)
				{
					pTmp++;
					memcpy(pFRUNames,pTmp,len);
					pFRUNames[len] = 0;
				}
				else
					strcpy((char*)pFRUNames,(char*)"\0");

//				printf("libipmi: FRUName = %s\n",pFRUNames);
				pFRUNames += 33;
				(*nFRUs)++;
			}
		}
		pSDRBuff += MaxSDRLen;
	}
	free(pSDRStart);
//	printf("numFRUs = %d\n",*nFRUs);
	return wRet;
}

uint8_t IPMICMD_GetSELTime(IPMI20_UDS_SESSION_T *pUDSSession,
				GetSELTimeRes_T *pGetSELTimeRes, int timeout)
{
	uint16_t	wRet;
	uint32_t	dwResLen;
	uint8_t	DummyReqData[8];

	dwResLen = sizeof(GetSELTimeRes_T);
	wRet = LIBIPMI_Send_RAW_IPMI2_0_Command(pUDSSession,  
						NETFNLUN_IPMI_STORAGE, CMD_GET_SEL_TIME,
						(uint8_t*)DummyReqData, 0,
						(uint8_t *)pGetSELTimeRes, &dwResLen,
						timeout);
	if (wRet != 0)
	{
		printf("Error getting SEL Time::%x\n",wRet);
	}
	return wRet;
}

uint8_t IPMC_GetSELTime(IPMI20_UDS_SESSION_T *pUDSSession, uint8_t IpmbAddr,
				GetSELTimeRes_T *pGetSELTimeRes, int timeout)
{
	int wRet = 0;
	uint32_t	dwResLen;
	uint8_t 	Req[50] = {0};
	uint8_t		Res[50] = {0};

	Req[0] = 0x40;	//Track | channel 0
	Req[1] = IpmbAddr;
	Req[2] = NETFNLUN_IPMI_STORAGE;
	Req[3] = 0x100 - (Req[1] + Req[2])&0xff;	//checksum1
	Req[4] = 0x20;
	Req[5] = 0x10;
	Req[6] = CMD_GET_SEL_TIME;	
	Req[7] = 0x100 - (Req[4] + Req[5] + Req[6])&0xff;

	wRet = LIBIPMI_Send_RAW_IPMI2_0_Command(pUDSSession,  
							NETFNLUN_IPMI_APP, CMD_SEND_MSG,
							Req, 8,
							Res,  &dwResLen,
							timeout);
	if ((wRet != 0) || (Res[6] != 0))
	{
		printf("IPMC Error getting SEL Time::%x\n",wRet);
		pGetSELTimeRes->CompletionCode = Res[6];
		pGetSELTimeRes->Time = 0;
		return wRet;
	}
	memcpy(pGetSELTimeRes, &Res[6], sizeof(GetSELTimeRes_T));
	return 0;
}

uint8_t IPMICMD_SetSELTime(IPMI20_UDS_SESSION_T *pUDSSession,
				SetSELTimeReq_T *pSetSELTimeReq, int timeout)
{
	uint16_t	wRet;
	uint32_t	dwResLen;
	uint8_t	DummyResData[8];

	dwResLen = sizeof(uint8_t);
	wRet = LIBIPMI_Send_RAW_IPMI2_0_Command(pUDSSession,  
						NETFNLUN_IPMI_STORAGE, CMD_SET_SEL_TIME,
						(uint8_t *)pSetSELTimeReq, sizeof(SetSELTimeReq_T),
						(uint8_t *)DummyResData, &dwResLen,
						timeout);
	if (wRet != 0)
	{
		printf("Error setting SEL Time::%x\n",wRet);
	}
	return wRet;
}

uint8_t IPMC_SetSELTime(IPMI20_UDS_SESSION_T *pUDSSession, uint8_t IpmbAddr,
				SetSELTimeReq_T *pSetSELTimeReq, int timeout)
{
	int wRet = 0;
	uint32_t	dwResLen;
	uint8_t 	Req[50] = {0};
	uint8_t		Res[50] = {0};

	Req[0] = 0x40;	//Track | channel 0
	Req[1] = IpmbAddr;
	Req[2] = NETFNLUN_IPMI_STORAGE;
	Req[3] = 0x100 - (Req[1] + Req[2])&0xff;	//checksum1
	Req[4] = 0x20;
	Req[5] = 0x10;
	Req[6] = CMD_SET_SEL_TIME;	
	Req[7] = pSetSELTimeReq->Time&0xff;
	Req[8] = (pSetSELTimeReq->Time>>8)&0xff;
	Req[9] = (pSetSELTimeReq->Time>>16)&0xff;
	Req[10] = (pSetSELTimeReq->Time>>24)&0xff;
	Req[11] = 0x100 - (Req[4] + Req[5] + Req[6] + Req[7] + Req[8] + Req[9] + Req[10])&0xff;

	wRet = LIBIPMI_Send_RAW_IPMI2_0_Command(pUDSSession,  
							NETFNLUN_IPMI_APP, CMD_SEND_MSG,
							Req, 12,
							Res,  &dwResLen,
							timeout);
	if ((wRet != 0) || (Res[6] != 0))
	{
		printf("Error setting SEL Time::%x\n",wRet);
	}
	return wRet;
}

uint8_t IPMICMD_GetSELTimeUTCOffset(IPMI20_UDS_SESSION_T *pUDSSession,
				GetSELTimeUTCOffsetRes_T *pGetSELTimeUTCOffsetRes, int timeout)
{
	uint16_t	wRet;
	uint32_t	dwResLen;
	uint8_t	DummyReqData[8];

	dwResLen = sizeof(GetSELTimeUTCOffsetRes_T);
	wRet = LIBIPMI_Send_RAW_IPMI2_0_Command(pUDSSession,  
						NETFNLUN_IPMI_STORAGE, CMD_GET_SEL_TIME_UTC_OFFSET,
						(uint8_t*)DummyReqData, 0,
						(uint8_t *)pGetSELTimeUTCOffsetRes, &dwResLen,
						timeout);
	if (wRet != 0)
	{
		printf("Error getting SEL Time UTC Offset::%x\n",wRet);
	}
	return wRet;
}

uint8_t IPMC_GetSELTimeUTCOffset(IPMI20_UDS_SESSION_T *pUDSSession, uint8_t IpmbAddr,
				GetSELTimeUTCOffsetRes_T *pGetSELTimeUTCOffsetRes, int timeout)
{
	int wRet = 0;
	uint32_t	dwResLen;
	uint8_t 	Req[50] = {0};
	uint8_t		Res[50] = {0};

	Req[0] = 0x40;	//Track | channel 0
	Req[1] = IpmbAddr;
	Req[2] = NETFNLUN_IPMI_STORAGE;
	Req[3] = 0x100 - (Req[1] + Req[2])&0xff;	//checksum1
	Req[4] = 0x20;
	Req[5] = 0x10;
	Req[6] = CMD_GET_SEL_TIME_UTC_OFFSET;	
	Req[7] = 0x100 - (Req[4] + Req[5] + Req[6])&0xff;

	wRet = LIBIPMI_Send_RAW_IPMI2_0_Command(pUDSSession,  
							NETFNLUN_IPMI_APP, CMD_SEND_MSG,
							Req, 8,
							Res,  &dwResLen,
							timeout);
	if ((wRet != 0) || (Res[6] != 0))
	{
		printf("Error getting SEL Time UTC Offset::%x\n",wRet);
		pGetSELTimeUTCOffsetRes->CompletionCode;
		pGetSELTimeUTCOffsetRes->UTCOffset = 0;
		return wRet;
	}
	memcpy(pGetSELTimeUTCOffsetRes, &Res[6], sizeof(GetSELTimeUTCOffsetRes_T));
	return 0;
}

uint8_t IPMICMD_SetSELTimeUTCOffset(IPMI20_UDS_SESSION_T *pUDSSession,
				SetSELTimeUTCOffsetReq_T *pSetSELTimeUTCOffsetReq, int timeout)
{
	uint16_t	wRet;
	uint32_t	dwResLen;
	SetSELTimeUTCOffsetRes_T pSetSELTimeUTCOffsetRes;

	dwResLen = sizeof(SetSELTimeUTCOffsetRes_T);
	wRet = LIBIPMI_Send_RAW_IPMI2_0_Command(pUDSSession,  
						NETFNLUN_IPMI_STORAGE, CMD_SET_SEL_TIME_UTC_OFFSET,
						(uint8_t *)pSetSELTimeUTCOffsetReq, sizeof(SetSELTimeUTCOffsetReq_T),
						(uint8_t *)&pSetSELTimeUTCOffsetRes, &dwResLen,
						timeout);
	if (wRet != 0)
	{
		printf("Error setting SEL Time UTC Offset::%x\n",wRet);
	}
	return wRet;
}

uint8_t IPMC_SetSELTimeUTCOffset(IPMI20_UDS_SESSION_T *pUDSSession, uint8_t IpmbAddr,
				SetSELTimeUTCOffsetReq_T *pSetSELTimeUTCOffsetReq, int timeout)
{
	int wRet = 0;
	uint32_t	dwResLen;
	uint8_t 	Req[50] = {0};
	uint8_t		Res[50] = {0};

	Req[0] = 0x40;	//Track | channel 0
	Req[1] = IpmbAddr;
	Req[2] = NETFNLUN_IPMI_STORAGE;
	Req[3] = 0x100 - (Req[1] + Req[2])&0xff;	//checksum1
	Req[4] = 0x20;
	Req[5] = 0x10;
	Req[6] = CMD_SET_SEL_TIME_UTC_OFFSET;	
	Req[7] = pSetSELTimeUTCOffsetReq->UTCOffset&0xff;
	Req[8] = (pSetSELTimeUTCOffsetReq->UTCOffset>>8)&0xff;
	Req[9] = 0x100 - (Req[4] + Req[5] + Req[6] + Req[7] + Req[8])&0xff;

	wRet = LIBIPMI_Send_RAW_IPMI2_0_Command(pUDSSession,  
							NETFNLUN_IPMI_APP, CMD_SEND_MSG,
							Req, 10,
							Res,  &dwResLen,
							timeout);
	if ((wRet != 0) || (Res[6] != 0))
	{
		printf("Error setting SEL Time UTC Offset::%x\n",wRet);
		return wRet;
	}
	return 0;
}